subroutine tise2d(N,kx,ky,Evec,Eval,V,xa,xb,ya,yb,t0,QPeps,QPforce)
  implicit none
  integer,intent(in)::N,QPforce,kx(1:N),ky(1:N)
  double precision,intent(in)::xa,xb,ya,yb,t0,QPeps
  double precision,intent(out)::Eval(1:N),Evec(1:N,1:N)

  double precision,external::V
  
  integer::i,j,k,prog
  double precision,parameter::pi=3.1415926535897932d0
  double precision::kinx,kiny,s

  Evec=0d0
  Eval=0d0

  !-----------------------------------------------
  ! Solve time-independent Schr\"odinger equation
  !-----------------------------------------------
  prog=0
  !$omp parallel do shared(prog) schedule(dynamic,1)
  do i=1,N
     do j=i,N
        call dqag4xy(V,kx(i),ky(i),kx(j),ky(j),t0,xa,xb,ya,yb,QPeps,QPforce,Evec(i,j))
     enddo
     prog=prog+1
     if(mod(prog,10).eq.0)write(6,'(A,i6,A,i6)')" ",prog," / ",N
  enddo
  !$omp end parallel do
  
  kinx=pi/(xb-xa)
  kiny=pi/(yb-ya)
  do i=1,N
     Evec(i,i) =  Evec(i,i) + 0.5d0*(kx(i)*kinx)**2 + 0.5d0*(ky(i)*kiny)**2 
  enddo 
    
  call diag(N,Evec,Eval)
  
  ! Normalize
  do k=1,N
     s=0d0
     do i=1,N
        s=s+Evec(i,k)*Evec(i,k)
     enddo
     Evec(1:N,k)=Evec(1:N,k)/sqrt(s)
  enddo
  
  return
end subroutine tise2d

subroutine diag(N,A,ev)
  implicit none
  integer,intent(in)::N
  double precision,intent(inout)::A(1:N,1:N)
  double precision,intent(out)::ev(1:N)
  !for Hermite matrix
 
  integer::lda,lwork,liwork,info
  double precision,allocatable::work(:)
  double precision::qw(1:3)
  integer,allocatable::iwork(:)
  integer::qiw(1:3)
  character(1)::job,uplo
 
  job="V"; uplo="U"; lda=N
 
  !size query of dsyevd or zheevd parameter.
  call dsyevd(job,uplo,N,0,lda,0,qw,-1,qiw,-1,info)
  if(info.ne.0)then
     write(6,'(A)')"    something wrong at dsyevd(size query) in diag"
     write(6,'(A)')"    program stop"
     write(6,'(A)')"    info --> ",info
     stop
  endif
 
  !just to be safe, +1
  lwork=idint(qw(1))+1; liwork=qiw(1)+1
  allocate(work(1:lwork),iwork(1:liwork))
  work(1:lwork)=0d0; iwork(1:liwork)=0
 
  !diagonalize.
  call dsyevd(job,uplo,N,A,lda,ev,work,lwork,iwork,liwork,info)
  if(info.ne.0)then
     write(6,'(A)')"    something wrong at subroutine dsyevd in diag"
     write(6,'(A)')"    program stop"
     write(6,'(A)')"    info --> ",info
     stop
  endif
 
  deallocate(work,iwork)
 
  return
end subroutine diag





!===================================   
   
subroutine dqag4xy(f,mx,my,nx,ny,t,xa,xb,ya,yb,eps,force,s)
  implicit none
  integer,intent(in)::mx,my,nx,ny,force
  double precision,intent(in)::t,xa,xb,ya,yb,eps
  double precision,intent(out)::s
  double precision,external::f
  !
  ! Quadpack integration package, using key=4
  !    (Gauss quadrature with 20-41 points)
  !
  integer::neval,limit,lenw,last,ier,l1,l2,l3,lvl,count
  integer,allocatable::iwork(:)
  double precision,allocatable::work(:)
  double precision::epsabs,abserr,xeps,yeps,a,b

  !=============
  limit=1000
  ! limit increase --> converge
  !=============

  s=0d0
  lenw=limit*4
  allocate(iwork(1:limit),work(1:lenw))
  !epsabs=-1d0
  epsabs=eps
  ier = 6
  neval = 0
  last = 0
  abserr = 0d0
  count = 0

  xeps=eps !sqrt(eps)
  yeps=eps

  a=xa
  b=xb
  
  l1 = limit+1
  l2 = limit+l1
  l3 = limit+l2
  call dqagexy(f,mx,my,nx,ny,t,a,b,xa,xb,ya,yb,yeps,epsabs,xeps,force,limit,s,abserr,neval,&
       ier,work(1),work(l1),work(l2),work(l3),iwork,last,count)
  lvl = 0
  if(ier.eq.6) lvl = 1
  if(ier.ne.0) call xerror("26habnormal return from dqag" ,26,ier,lvl)
 
  ! ier = 0 : success, converged.
  ! ier > 0 : fail

  !write(6,*)count,ier ! Number of times called integrand function

  if(ier.eq.1)then
     write(6,'(A,6i5)')"Try the parameter 'limit' increase.",mx,my,nx,ny,count,ier
  endif

  deallocate(iwork,work)
  return
end subroutine dqag4xy

subroutine dqagexy(f,mx,my,nx,ny,t,a,b,xa,xb,ya,yb,yeps,epsabs,epsrel,force,limit,result,abserr, &
     neval,ier,alist,blist,rlist,elist,iord,last,count)
  !***author  piessens,robert,appl. math. & progr. div - k.u.leuven
  !           de doncker,elise,appl. math. & progr. div. - k.u.leuven
  !   2018/07/07 modified by sikino, http://slpr.sakura.ne.jp/qp/
  double precision a,abserr,alist,area,area1,area12,area2,a1,a2,b,&
       blist,b1,b2,dabs,defabs,defab1,defab2,dmax1,elist,epmach,&
       epsabs,epsrel,errbnd,errmax,error1,error2,erro12,errsum,f,&
       resabs,result,rlist,uflow,t,xa,xb,ya,yb,yeps
  integer ier,iord,iroff1,iroff2,k,last,limit,maxerr,neval,&
       nrmax,igt,igk,count,mx,my,nx,ny,force
  dimension alist(limit),blist(limit),elist(limit),iord(limit),&
       rlist(limit)
  external f
  epmach = epsilon(a)
  uflow = tiny(a)
  ier = 0
  neval = 0
  last = 0
  result = 0.0d+00
  abserr = 0.0d+00
  alist(1) = a
  blist(1) = b
  rlist(1) = 0.0d+00
  elist(1) = 0.0d+00
  iord(1) = 0
  if(epsabs.le.0.0d+00.and.&
       epsrel.lt.dmax1(0.5d+02*epmach,0.5d-28)) ier = 6
  if(ier.ne.6)then
     neval = 0
     call dqk41xy(f,mx,my,nx,ny,t,a,b,xa,xb,ya,yb,yeps,result,abserr,defabs,resabs,force,count)
     last = 1
     rlist(1) = result
     elist(1) = abserr
     iord(1) = 1
     errbnd = dmax1(epsabs,epsrel*dabs(result))
     if(abserr.le.0.5d+02*epmach*defabs.and.abserr.gt.errbnd) ier = 2
     if(limit.eq.1) ier = 1
     igk=0

     if(force.eq.1)then
        if(ier.ne.0)igk=60        
     else
        if(ier.ne.0.or.(abserr.le.errbnd.and.abserr.ne.resabs)&
             .or.abserr.eq.0.0d+00)igk=60
     endif

     if(igk.ne.60)then
        errmax = abserr
        maxerr = 1
        area = result
        errsum = abserr
        nrmax = 1
        iroff1 = 0
        iroff2 = 0
        do last = 2,limit
           a1 = alist(maxerr)
           b1 = 0.5d+00*(alist(maxerr)+blist(maxerr))
           a2 = b1
           b2 = blist(maxerr)
           call dqk41xy(f,mx,my,nx,ny,t,a1,b1,xa,xb,ya,yb,yeps,area1,error1,resabs,defab1,force,count)
           call dqk41xy(f,mx,my,nx,ny,t,a2,b2,xa,xb,ya,yb,yeps,area2,error2,resabs,defab2,force,count)
           neval = neval+1
           area12 = area1+area2
           erro12 = error1+error2
           errsum = errsum+erro12-errmax
           area = area+area12-rlist(maxerr)
           if(defab1.ne.error1.and.defab2.ne.error2)then
              if(dabs(rlist(maxerr)-area12).le.0.1d-04*dabs(area12)&
                   .and.erro12.ge.0.99d+00*errmax) iroff1 = iroff1+1
              if(last.gt.10.and.erro12.gt.errmax) iroff2 = iroff2+1
           endif
           rlist(maxerr) = area1
           rlist(last) = area2
           errbnd = dmax1(epsabs,epsrel*dabs(area))
           if(errsum.gt.errbnd)then
              if(iroff1.ge.6.or.iroff2.ge.20) ier = 2
              if(last.eq.limit) ier = 1
              if(dmax1(dabs(a1),dabs(b2)).le.(0.1d+01+0.1d+03* &
                   epmach)*(dabs(a2)+0.1d+04*uflow)) ier = 3
           endif
           igt=0
           if(error2.le.error1)then
              alist(last) = a2
              blist(maxerr) = b1
              blist(last) = b2
              elist(maxerr) = error1
              elist(last) = error2
              igt=20
           endif
           if(igt.ne.20)then
              alist(maxerr) = a2
              alist(last) = a1
              blist(last) = b1
              rlist(maxerr) = area2
              rlist(last) = area1
              elist(maxerr) = error2
              elist(last) = error1
           endif
           call dqpsrt(limit,last,maxerr,errmax,elist,iord,nrmax)
           if(ier.ne.0.or.errsum.le.errbnd)exit
        enddo
        if(last.eq.limit+1)last=last-1
        result = 0.0d+00
        do k=1,last
           result = result+rlist(k)
        enddo
        abserr = errsum
     endif
  end if
  return
end subroutine dqagexy

subroutine dqk41xy(f,mx,my,nx,ny,t,a,b,xa,xb,ya,yb,yeps,result,abserr,resabs,resasc,force,count)
  !***author  piessens,robert,appl. math. & progr. div - k.u.leuven
  !           de doncker,elise,appl. math. & progr. div. - k.u.leuven
  !   2018/07/07 modified by sikino, http://slpr.sakura.ne.jp/qp/
  double precision a,absc,abserr,b,centr,dabs,dhlgth,dmax1,dmin1,&
       epmach,f,fc,fsum,fval1,fval2,fv1,fv2,hlgth,resabs,resasc,&
       resg,resk,reskh,result,uflow,wg,wgk,xgk,t,xa,xb,ya,yb,yeps,&
       sn,sm,samp,piab,x
  double precision,parameter::pi=3.14159265358979324d0
  integer j,jtw,jtwm1,count,mx,my,nx,ny,force
  external f
  dimension fv1(20),fv2(20),xgk(21),wgk(21),wg(10)
  data wg  (  1) / 0.017614007139152118311861962351853d0 /
  data wg  (  2) / 0.040601429800386941331039952274932d0 /
  data wg  (  3) / 0.062672048334109063569506535187042d0 /
  data wg  (  4) / 0.083276741576704748724758143222046d0 /
  data wg  (  5) / 0.101930119817240435036750135480350d0 /
  data wg  (  6) / 0.118194531961518417312377377711382d0 /
  data wg  (  7) / 0.131688638449176626898494499748163d0 /
  data wg  (  8) / 0.142096109318382051329298325067165d0 /
  data wg  (  9) / 0.149172986472603746787828737001969d0 /
  data wg  ( 10) / 0.152753387130725850698084331955098d0 /
  !
  data xgk (  1) / 0.998859031588277663838315576545863d0 /
  data xgk (  2) / 0.993128599185094924786122388471320d0 /
  data xgk (  3) / 0.981507877450250259193342994720217d0 /
  data xgk (  4) / 0.963971927277913791267666131197277d0 /
  data xgk (  5) / 0.940822633831754753519982722212443d0 /
  data xgk (  6) / 0.912234428251325905867752441203298d0 /
  data xgk (  7) / 0.878276811252281976077442995113078d0 /
  data xgk (  8) / 0.839116971822218823394529061701521d0 /
  data xgk (  9) / 0.795041428837551198350638833272788d0 /
  data xgk ( 10) / 0.746331906460150792614305070355642d0 /
  data xgk ( 11) / 0.693237656334751384805490711845932d0 /
  data xgk ( 12) / 0.636053680726515025452836696226286d0 /
  data xgk ( 13) / 0.575140446819710315342946036586425d0 /
  data xgk ( 14) / 0.510867001950827098004364050955251d0 /
  data xgk ( 15) / 0.443593175238725103199992213492640d0 /
  data xgk ( 16) / 0.373706088715419560672548177024927d0 /
  data xgk ( 17) / 0.301627868114913004320555356858592d0 /
  data xgk ( 18) / 0.227785851141645078080496195368575d0 /
  data xgk ( 19) / 0.152605465240922675505220241022678d0 /
  data xgk ( 20) / 0.076526521133497333754640409398838d0 /
  data xgk ( 21) / 0.000000000000000000000000000000000d0 /
  !
  data wgk (  1) / 0.003073583718520531501218293246031d0 /
  data wgk (  2) / 0.008600269855642942198661787950102d0 /
  data wgk (  3) / 0.014626169256971252983787960308868d0 /
  data wgk (  4) / 0.020388373461266523598010231432755d0 /
  data wgk (  5) / 0.025882133604951158834505067096153d0 /
  data wgk (  6) / 0.031287306777032798958543119323801d0 /
  data wgk (  7) / 0.036600169758200798030557240707211d0 /
  data wgk (  8) / 0.041668873327973686263788305936895d0 /
  data wgk (  9) / 0.046434821867497674720231880926108d0 /
  data wgk ( 10) / 0.050944573923728691932707670050345d0 /
  data wgk ( 11) / 0.055195105348285994744832372419777d0 /
  data wgk ( 12) / 0.059111400880639572374967220648594d0 /
  data wgk ( 13) / 0.062653237554781168025870122174255d0 /
  data wgk ( 14) / 0.065834597133618422111563556969398d0 /
  data wgk ( 15) / 0.068648672928521619345623411885368d0 /
  data wgk ( 16) / 0.071054423553444068305790361723210d0 /
  data wgk ( 17) / 0.073030690332786667495189417658913d0 /
  data wgk ( 18) / 0.074582875400499188986581418362488d0 /
  data wgk ( 19) / 0.075704497684556674659542775376617d0 /
  data wgk ( 20) / 0.076377867672080736705502835038061d0 /
  data wgk ( 21) / 0.076600711917999656445049901530102d0 /

  epmach = epsilon(a)
  uflow = tiny(a)
  centr = 0.5d+00*(a+b)
  hlgth = 0.5d+00*(b-a)
  dhlgth = dabs(hlgth)
  resg = 0.0d+00

  samp=sqrt(2d0/(xb-xa))
  piab=pi/(xb-xa)

  !,mx,my,nx,ny,t
  !fc = f(centr) ! *************
  x=centr
  sm = samp*sin(mx*piab*(x-xa))! ***
  sn = samp*sin(nx*piab*(x-xa))! ***
  call dqag4y(f,my,ny,t,ya,yb,x,yeps,force,fc,count)
  fc = sm*fc*sn ! ***

  resk = wgk(21)*fc
  resabs = dabs(resk)
  do j=1,10
     jtw = j*2
     absc = hlgth*xgk(jtw)
     !fval1 = f(centr-absc) ! *************
     !fval2 = f(centr+absc) ! *************
     !call dqag4y(f,my,ny,t,ya,yb,centr-absc,yeps,fval1,count)
     !call dqag4y(f,my,ny,t,ya,yb,centr+absc,yeps,fval2,count)

     x=centr-absc
     sm = samp*sin(mx*piab*(x-xa))! ***
     sn = samp*sin(nx*piab*(x-xa))! ***
     call dqag4y(f,my,ny,t,ya,yb,x,yeps,force,fval1,count)
     fval1 = sm*fval1*sn ! ***

     x=centr+absc
     sm = samp*sin(mx*piab*(x-xa))! ***
     sn = samp*sin(nx*piab*(x-xa))! ***
     call dqag4y(f,my,ny,t,ya,yb,x,yeps,force,fval2,count)
     fval2 = sm*fval2*sn ! ***



     fv1(jtw) = fval1
     fv2(jtw) = fval2
     fsum = fval1+fval2
     resg = resg+wg(j)*fsum
     resk = resk+wgk(jtw)*fsum
     resabs = resabs+wgk(jtw)*(dabs(fval1)+dabs(fval2))
  enddo
  do j = 1,10
     jtwm1 = j*2-1
     absc = hlgth*xgk(jtwm1)
     !fval1 = f(centr-absc) ! *************
     !fval2 = f(centr+absc) ! *************
     !call dqag4y(f,my,ny,t,ya,yb,centr-absc,yeps,fval1,count)
     !call dqag4y(f,my,ny,t,ya,yb,centr+absc,yeps,fval2,count)

     x=centr-absc
     sm = samp*sin(mx*piab*(x-xa))! ***
     sn = samp*sin(nx*piab*(x-xa))! ***
     call dqag4y(f,my,ny,t,ya,yb,x,yeps,force,fval1,count)
     fval1 = sm*fval1*sn ! ***

     x=centr+absc
     sm = samp*sin(mx*piab*(x-xa))! ***
     sn = samp*sin(nx*piab*(x-xa))! ***
     call dqag4y(f,my,ny,t,ya,yb,x,yeps,force,fval2,count)
     fval2 = sm*fval2*sn ! ***

     fv1(jtwm1) = fval1
     fv2(jtwm1) = fval2
     fsum = fval1+fval2
     resk = resk+wgk(jtwm1)*fsum
     resabs = resabs+wgk(jtwm1)*(dabs(fval1)+dabs(fval2))
  enddo
  reskh = resk*0.5d+00
  resasc = wgk(21)*dabs(fc-reskh)
  do j=1,20
     resasc = resasc+wgk(j)*(dabs(fv1(j)-reskh)+dabs(fv2(j)-reskh))
  enddo
  result = resk*hlgth
  resabs = resabs*dhlgth
  resasc = resasc*dhlgth
  abserr = dabs((resk-resg)*hlgth)
  if(resasc.ne.0.0d+00.and.abserr.ne.0.d+00)&
       abserr = resasc*dmin1(0.1d+01,(0.2d+03*abserr/resasc)**1.5d+00)
  if(resabs.gt.uflow/(0.5d+02*epmach)) abserr = dmax1&
       ((epmach*0.5d+02)*resabs,abserr)
  count=count+41
  return
end subroutine dqk41xy

   
subroutine dqag4y(f,my,ny,t,ya,yb,xc,eps,force,s,count)
  implicit none
  integer,intent(in)::my,ny,force
  double precision,intent(in)::t,ya,yb,eps,xc
  double precision,intent(out)::s
  integer,intent(inout)::count
  double precision,external::f
  !
  ! Quadpack integration package, using key=4
  !    (Gauss quadrature with 20-41 points)
  !
  integer::neval,limit,lenw,last,ier,l1,l2,l3,lvl
  integer,allocatable::iwork(:)
  double precision,allocatable::work(:)
  double precision::epsabs,abserr,a,b

  !=============
  limit=1000
  ! limit increase --> converge
  !=============

  s=0d0
  lenw=limit*4
  allocate(iwork(1:limit),work(1:lenw))
  !epsabs=-1d0
  epsabs=eps
  ier = 6
  neval = 0
  last = 0
  abserr = 0d0
!  count = 0

  a=ya
  b=yb
  
  l1 = limit+1
  l2 = limit+l1
  l3 = limit+l2
  call dqagey(f,my,ny,t,a,b,ya,yb,xc,epsabs,eps,force,limit,s,abserr,neval,&
       ier,work(1),work(l1),work(l2),work(l3),iwork,last,count)
  lvl = 0
  if(ier.eq.6) lvl = 1
  if(ier.ne.0) call xerror("26habnormal return from dqag" ,26,ier,lvl)
 
  ! ier = 0 : success, converged.
  ! ier > 0 : fail

  !write(6,*)count,ier ! Number of times called integrand function

  if(ier.eq.1)then
     write(6,*)"Try the parameter 'limit' increase."
  endif

  deallocate(iwork,work)
  return
end subroutine dqag4y

subroutine dqagey(f,my,ny,t,a,b,ya,yb,xc,epsabs,epsrel,force,limit,result,abserr, &
     neval,ier,alist,blist,rlist,elist,iord,last,count)
  !***author  piessens,robert,appl. math. & progr. div - k.u.leuven
  !           de doncker,elise,appl. math. & progr. div. - k.u.leuven
  !   2018/07/07 modified by sikino, http://slpr.sakura.ne.jp/qp/
  double precision a,abserr,alist,area,area1,area12,area2,a1,a2,b,&
       blist,b1,b2,dabs,defabs,defab1,defab2,dmax1,elist,epmach,&
       epsabs,epsrel,errbnd,errmax,error1,error2,erro12,errsum,f,&
       resabs,result,rlist,uflow,xc,ya,yb,t
  integer ier,iord,iroff1,iroff2,k,last,limit,maxerr,neval,&
       nrmax,igt,igk,count,my,ny,force
  dimension alist(limit),blist(limit),elist(limit),iord(limit),&
       rlist(limit)
  external f
  epmach = epsilon(a)
  uflow = tiny(a)
  ier = 0
  neval = 0
  last = 0
  result = 0.0d+00
  abserr = 0.0d+00
  alist(1) = a
  blist(1) = b
  rlist(1) = 0.0d+00
  elist(1) = 0.0d+00
  iord(1) = 0
  if(epsabs.le.0.0d+00.and.&
       epsrel.lt.dmax1(0.5d+02*epmach,0.5d-28)) ier = 6
  if(ier.ne.6)then
     neval = 0
     call dqk41y(f,my,ny,t,a,b,ya,yb,xc,result,abserr,defabs,resabs,count)
     last = 1
     rlist(1) = result
     elist(1) = abserr
     iord(1) = 1
     errbnd = dmax1(epsabs,epsrel*dabs(result))
     if(abserr.le.0.5d+02*epmach*defabs.and.abserr.gt.errbnd) ier = 2
     if(limit.eq.1) ier = 1
     igk=0

     if(force.eq.1)then
        if(ier.ne.0)igk=60        
     else
        if(ier.ne.0.or.(abserr.le.errbnd.and.abserr.ne.resabs)&
             .or.abserr.eq.0.0d+00)igk=60
     endif

     if(igk.ne.60)then
        errmax = abserr
        maxerr = 1
        area = result
        errsum = abserr
        nrmax = 1
        iroff1 = 0
        iroff2 = 0
        do last = 2,limit
           a1 = alist(maxerr)
           b1 = 0.5d+00*(alist(maxerr)+blist(maxerr))
           a2 = b1
           b2 = blist(maxerr)
           call dqk41y(f,my,ny,t,a1,b1,ya,yb,xc,area1,error1,resabs,defab1,count)
           call dqk41y(f,my,ny,t,a2,b2,ya,yb,xc,area2,error2,resabs,defab2,count)
           neval = neval+1
           area12 = area1+area2
           erro12 = error1+error2
           errsum = errsum+erro12-errmax
           area = area+area12-rlist(maxerr)
           if(defab1.ne.error1.and.defab2.ne.error2)then
              if(dabs(rlist(maxerr)-area12).le.0.1d-04*dabs(area12)&
                   .and.erro12.ge.0.99d+00*errmax) iroff1 = iroff1+1
              if(last.gt.10.and.erro12.gt.errmax) iroff2 = iroff2+1
           endif
           rlist(maxerr) = area1
           rlist(last) = area2
           errbnd = dmax1(epsabs,epsrel*dabs(area))
           if(errsum.gt.errbnd)then
              if(iroff1.ge.6.or.iroff2.ge.20) ier = 2
              if(last.eq.limit) ier = 1
              if(dmax1(dabs(a1),dabs(b2)).le.(0.1d+01+0.1d+03* &
                   epmach)*(dabs(a2)+0.1d+04*uflow)) ier = 3
           endif
           igt=0
           if(error2.le.error1)then
              alist(last) = a2
              blist(maxerr) = b1
              blist(last) = b2
              elist(maxerr) = error1
              elist(last) = error2
              igt=20
           endif
           if(igt.ne.20)then
              alist(maxerr) = a2
              alist(last) = a1
              blist(last) = b1
              rlist(maxerr) = area2
              rlist(last) = area1
              elist(maxerr) = error2
              elist(last) = error1
           endif
           call dqpsrt(limit,last,maxerr,errmax,elist,iord,nrmax)
           if(ier.ne.0.or.errsum.le.errbnd)exit
        enddo
        if(last.eq.limit+1)last=last-1
        result = 0.0d+00
        do k=1,last
           result = result+rlist(k)
        enddo
        abserr = errsum
     endif
  end if
  return
end subroutine dqagey

subroutine dqk41y(f,my,ny,t,a,b,ya,yb,xc,result,abserr,resabs,resasc,count)
  !***author  piessens,robert,appl. math. & progr. div - k.u.leuven
  !           de doncker,elise,appl. math. & progr. div. - k.u.leuven
  !   2018/07/07 modified by sikino, http://slpr.sakura.ne.jp/qp/
  double precision a,absc,abserr,b,centr,dabs,dhlgth,dmax1,dmin1,&
       epmach,f,fc,fsum,fval1,fval2,fv1,fv2,hlgth,resabs,resasc,&
       resg,resk,reskh,result,uflow,wg,wgk,xgk,xc,ya,yb,t,&
       samp,piab,sm,sn,y
  double precision,parameter::pi=3.14159265358979324d0
  integer j,jtw,jtwm1,count,my,ny
  external f
  dimension fv1(20),fv2(20),xgk(21),wgk(21),wg(10)
  data wg  (  1) / 0.017614007139152118311861962351853d0 /
  data wg  (  2) / 0.040601429800386941331039952274932d0 /
  data wg  (  3) / 0.062672048334109063569506535187042d0 /
  data wg  (  4) / 0.083276741576704748724758143222046d0 /
  data wg  (  5) / 0.101930119817240435036750135480350d0 /
  data wg  (  6) / 0.118194531961518417312377377711382d0 /
  data wg  (  7) / 0.131688638449176626898494499748163d0 /
  data wg  (  8) / 0.142096109318382051329298325067165d0 /
  data wg  (  9) / 0.149172986472603746787828737001969d0 /
  data wg  ( 10) / 0.152753387130725850698084331955098d0 /
  !
  data xgk (  1) / 0.998859031588277663838315576545863d0 /
  data xgk (  2) / 0.993128599185094924786122388471320d0 /
  data xgk (  3) / 0.981507877450250259193342994720217d0 /
  data xgk (  4) / 0.963971927277913791267666131197277d0 /
  data xgk (  5) / 0.940822633831754753519982722212443d0 /
  data xgk (  6) / 0.912234428251325905867752441203298d0 /
  data xgk (  7) / 0.878276811252281976077442995113078d0 /
  data xgk (  8) / 0.839116971822218823394529061701521d0 /
  data xgk (  9) / 0.795041428837551198350638833272788d0 /
  data xgk ( 10) / 0.746331906460150792614305070355642d0 /
  data xgk ( 11) / 0.693237656334751384805490711845932d0 /
  data xgk ( 12) / 0.636053680726515025452836696226286d0 /
  data xgk ( 13) / 0.575140446819710315342946036586425d0 /
  data xgk ( 14) / 0.510867001950827098004364050955251d0 /
  data xgk ( 15) / 0.443593175238725103199992213492640d0 /
  data xgk ( 16) / 0.373706088715419560672548177024927d0 /
  data xgk ( 17) / 0.301627868114913004320555356858592d0 /
  data xgk ( 18) / 0.227785851141645078080496195368575d0 /
  data xgk ( 19) / 0.152605465240922675505220241022678d0 /
  data xgk ( 20) / 0.076526521133497333754640409398838d0 /
  data xgk ( 21) / 0.000000000000000000000000000000000d0 /
  !
  data wgk (  1) / 0.003073583718520531501218293246031d0 /
  data wgk (  2) / 0.008600269855642942198661787950102d0 /
  data wgk (  3) / 0.014626169256971252983787960308868d0 /
  data wgk (  4) / 0.020388373461266523598010231432755d0 /
  data wgk (  5) / 0.025882133604951158834505067096153d0 /
  data wgk (  6) / 0.031287306777032798958543119323801d0 /
  data wgk (  7) / 0.036600169758200798030557240707211d0 /
  data wgk (  8) / 0.041668873327973686263788305936895d0 /
  data wgk (  9) / 0.046434821867497674720231880926108d0 /
  data wgk ( 10) / 0.050944573923728691932707670050345d0 /
  data wgk ( 11) / 0.055195105348285994744832372419777d0 /
  data wgk ( 12) / 0.059111400880639572374967220648594d0 /
  data wgk ( 13) / 0.062653237554781168025870122174255d0 /
  data wgk ( 14) / 0.065834597133618422111563556969398d0 /
  data wgk ( 15) / 0.068648672928521619345623411885368d0 /
  data wgk ( 16) / 0.071054423553444068305790361723210d0 /
  data wgk ( 17) / 0.073030690332786667495189417658913d0 /
  data wgk ( 18) / 0.074582875400499188986581418362488d0 /
  data wgk ( 19) / 0.075704497684556674659542775376617d0 /
  data wgk ( 20) / 0.076377867672080736705502835038061d0 /
  data wgk ( 21) / 0.076600711917999656445049901530102d0 /

  epmach = epsilon(a)
  uflow = tiny(a)
  centr = 0.5d+00*(a+b)
  hlgth = 0.5d+00*(b-a)
  dhlgth = dabs(hlgth)
  resg = 0.0d+00

  samp=sqrt(2d0/(yb-ya))
  piab=pi/(yb-ya)

  ! fc = f(xc,centr) ! *************
  y=centr
  sm = samp*sin(my*piab*(y-ya))! ***
  sn = samp*sin(ny*piab*(y-ya))! ***
  fc = sm*f(xc,y,t)*sn ! ***

  resk = wgk(21)*fc
  resabs = dabs(resk)
  do j=1,10
     jtw = j*2
     absc = hlgth*xgk(jtw)
     !fval1 = f(xc,centr-absc) ! *************
     !fval2 = f(xc,centr+absc) ! *************

     y=centr-absc
     sm = samp*sin(my*piab*(y-ya))! ***
     sn = samp*sin(ny*piab*(y-ya))! ***
     fval1 = sm*f(xc,y,t)*sn ! ***

     y=centr+absc
     sm = samp*sin(my*piab*(y-ya))! ***
     sn = samp*sin(ny*piab*(y-ya))! ***
     fval2 = sm*f(xc,y,t)*sn ! ***

     fv1(jtw) = fval1
     fv2(jtw) = fval2
     fsum = fval1+fval2
     resg = resg+wg(j)*fsum
     resk = resk+wgk(jtw)*fsum
     resabs = resabs+wgk(jtw)*(dabs(fval1)+dabs(fval2))
  enddo
  do j = 1,10
     jtwm1 = j*2-1
     absc = hlgth*xgk(jtwm1)
     !fval1 = f(xc,centr-absc) ! *************
     !fval2 = f(xc,centr+absc) ! *************
     
     y=centr-absc
     sm = samp*sin(my*piab*(y-ya))! ***
     sn = samp*sin(ny*piab*(y-ya))! ***
     fval1 = sm*f(xc,y,t)*sn ! ***

     y=centr+absc
     sm = samp*sin(my*piab*(y-ya))! ***
     sn = samp*sin(ny*piab*(y-ya))! ***
     fval2 = sm*f(xc,y,t)*sn ! ***


     fv1(jtwm1) = fval1
     fv2(jtwm1) = fval2
     fsum = fval1+fval2
     resk = resk+wgk(jtwm1)*fsum
     resabs = resabs+wgk(jtwm1)*(dabs(fval1)+dabs(fval2))
  enddo
  reskh = resk*0.5d+00
  resasc = wgk(21)*dabs(fc-reskh)
  do j=1,20
     resasc = resasc+wgk(j)*(dabs(fv1(j)-reskh)+dabs(fv2(j)-reskh))
  enddo
  result = resk*hlgth
  resabs = resabs*dhlgth
  resasc = resasc*dhlgth
  abserr = dabs((resk-resg)*hlgth)
  if(resasc.ne.0.0d+00.and.abserr.ne.0.d+00)&
       abserr = resasc*dmin1(0.1d+01,(0.2d+03*abserr/resasc)**1.5d+00)
  if(resabs.gt.uflow/(0.5d+02*epmach)) abserr = dmax1&
       ((epmach*0.5d+02)*resabs,abserr)
  count=count+41
  return
end subroutine dqk41y


























subroutine dqpsrt(limit,last,maxerr,ermax,elist,iord,nrmax)
  double precision elist,ermax,errmax,errmin
  integer i,ibeg,ido,iord,isucc,j,jbnd,jupbn,k,last,limit,maxerr,&
       nrmax,igt,igt1,igt2
  dimension elist(last),iord(last)
  igt=0
  if(last.le.2)then
     iord(1) = 1
     iord(2) = 2
     igt=90
  endif
  if(igt.ne.90)then
     errmax = elist(maxerr)
     if(nrmax.ne.1)then
        ido = nrmax-1
        do i = 1,ido
           isucc = iord(nrmax-1)
           if(errmax.le.elist(isucc))exit
           iord(nrmax) = isucc
           nrmax = nrmax-1
        enddo
     endif
     jupbn = last
     if(last.gt.(limit/2+2)) jupbn = limit+3-last
     errmin = elist(last)
     jbnd = jupbn-1
     ibeg = nrmax+1
     igt1=0
     if(ibeg.le.jbnd)then
        do i=ibeg,jbnd
           isucc = iord(i)
           if(errmax.ge.elist(isucc))then
              igt1=60
              exit
           endif
           iord(i-1) = isucc
        enddo
     endif
     if(igt1.ne.60)then
        iord(jbnd) = maxerr
        iord(jupbn) = last
        igt=90
     endif
     igt2=0
     if(igt.ne.90)then
        iord(i-1) = maxerr
        k = jbnd
        do j=i,jbnd
           isucc = iord(k)
           if(errmin.lt.elist(isucc))then
              igt2=80
              exit
           endif
           iord(k+1) = isucc
           k = k-1
        enddo
        if(igt2.ne.80)then
           iord(i) = last
           igt=90
        endif
        if(igt.ne.90)then
           iord(k+1) = last
        endif
     endif
  endif
  maxerr = iord(nrmax)
  ermax = elist(maxerr)
  return
end subroutine dqpsrt

subroutine xerror ( xmess, nmess, nerr, level )
  implicit none
  integer::level,nerr,nmess
  character ( len = * ) xmess  
  if ( 1 <= LEVEL ) then
     WRITE ( *,'(1X,A)') XMESS(1:NMESS)
     WRITE ( *,'('' ERROR NUMBER = '',I5,'', MESSAGE LEVEL = '',I5)') &
          NERR,LEVEL
  end if
  return
end subroutine xerror
